@@ -42,7 +42,7 @@ AMQP_PROTOCOL_HEADER = 'AMQP\x01\x01\x09\x01'
 #
 LIBRARY_PROPERTIES = {
     'library': 'Python amqplib',
-    'library_version': '0.1',
+    'library_version': '0.2',
     }
 
 DEBUG = False
@@ -111,7 +111,7 @@ class Connection(object):
     def __init__(self, host, userid=None, password=None,
         login_method='AMQPLAIN', login_response=None,
         virtual_host='/', locale='en_US', client_properties={},
-        ssl=False, **kwargs):
+        ssl=False, insist=False, **kwargs):
         """
         Create a connection to the specified host, which should be
         a 'host[:port]', such as 'localhost', or '1.2.3.4:5672'
@@ -120,33 +120,6 @@ class Connection(object):
         for you.  Otherwise you have to roll your own.
 
         """
-        self.channels = {}
-        self.frame_queue = Queue()
-        self.input = self.out = None
-
-        if ':' in host:
-            host, port = host.split(':', 1)
-            port = int(port)
-        else:
-            port = AMQP_PORT
-
-        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        sock.connect((host, port))
-
-        if ssl:
-            self.out = _SSLWrap(sock)
-            self.input = AMQPReader(self.out)
-        else:
-            self.out = sock.makefile('w')
-            self.input = AMQPReader(sock.makefile('r'))
-
-        self.out.write(AMQP_PROTOCOL_HEADER)
-        self.out.flush()
-
-        self.wait(allowed_methods=[
-                (10, 10), # start
-                ])
-
         if (userid is not None) and (password is not None):
             login_response = AMQPWriter()
             login_response.write_table({'LOGIN': userid, 'PASSWORD': password})
@@ -156,22 +129,60 @@ class Connection(object):
         d = {}
         d.update(LIBRARY_PROPERTIES)
         d.update(client_properties)
-        self._x_start_ok(d, login_method, login_response, locale)
 
-        self._wait_tune_ok = True
-        while self._wait_tune_ok:
+        self.known_hosts = ''
+
+        while True:
+            self.channels = {}
+            self.frame_queue = Queue()
+            self.input = self.out = None
+
+            if ':' in host:
+                host, port = host.split(':', 1)
+                port = int(port)
+            else:
+                port = AMQP_PORT
+
+            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sock.connect((host, port))
+
+            if ssl:
+                self.out = _SSLWrap(sock)
+                self.input = AMQPReader(self.out)
+            else:
+                self.out = sock.makefile('w')
+                self.input = AMQPReader(sock.makefile('r'))
+
+            self.out.write(AMQP_PROTOCOL_HEADER)
+            self.out.flush()
+
             self.wait(allowed_methods=[
-                (10, 20), # secure
-                (10, 30), # tune
-                ])
+                    (10, 10), # start
+                    ])
 
-        self._x_open(virtual_host)
+            self._x_start_ok(d, login_method, login_response, locale)
+
+            self._wait_tune_ok = True
+            while self._wait_tune_ok:
+                self.wait(allowed_methods=[
+                    (10, 20), # secure
+                    (10, 30), # tune
+                    ])
+
+            host = self._x_open(virtual_host, insist=insist)
+            if host is None:
+                # we weren't redirected
+                return
+
+            # we were redirected, close the socket, loop and try again
+            self.close()
 
 
     def __del__(self):
         if self.input is not None:
             self.close()
 
+
     def _do_close(self):
         self.input.close()
         self.out.close()
@@ -183,6 +194,7 @@ class Connection(object):
             return self.frame_queue
         return self.channels[channel_id].frame_queue
 
+
     def _get_free_channel_id(self):
         for i in xrange(1, self.channel_max+1):
             if i not in self.channels:
@@ -433,6 +445,7 @@ class Connection(object):
         self.known_hosts = args.read_shortstr()
         if DEBUG:
             print 'Open OK! known_hosts [%s]' % self.known_hosts
+        return None
 
 
     def _redirect(self, args):
@@ -442,7 +455,10 @@ class Connection(object):
 
         """
         host = args.read_shortstr()
-        known_hosts = args.read_shortstr()
+        self.known_hosts = args.read_shortstr()
+        if DEBUG:
+            print 'Redirected to [%s], known_hosts [%s]' % (host, self.known_hosts)
+        return host
 
 
     def _secure(self, args):
@@ -853,7 +869,7 @@ class Channel(object):
     #
 
     def exchange_declare(self, exchange, type, passive=False, durable=False,
-        auto_delete=False, internal=False, nowait=False,
+        auto_delete=True, internal=False, nowait=False,
         arguments={}, ticket=None):
         """
         This method creates an exchange if it does not already exist, and if
@@ -967,7 +983,7 @@ class Channel(object):
 
 
     def queue_declare(self, queue='', passive=False, durable=False,
-        exclusive=False, auto_delete=False, nowait=False,
+        exclusive=False, auto_delete=True, nowait=False,
         arguments={}, ticket=None):
         """
         This method creates or checks a queue.  When creating a new queue
